// import third party packages
const createError = require('http-errors');
const express = require('express');
const path = require('path');
const cookieParser = require('cookie-parser');
const logger = require('morgan');

// import modules
const connectionRequestHandler = require('./connection-request-handler');
const webhookPayloadHandler = require('./webhook-payload-handler');

// create express app, initialize miscellaneous middleware
const app = express();
app.use(logger('dev'));
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(function(req, res, next) {
    console.log(req.body);
    next();
});
app.use(cookieParser());

// send frontend javascript, html, css to browser when requested
app.use(express.static(path.join(__dirname, '../public')));

// handle connection creation request from frontend
app.post('/connections/add', (req, res) => {
    res.json(connectionRequestHandler.add(req.body));
});

// handle connection edit request from frontend
app.post('/connections/edit', (req, res) => {
    res.json(connectionRequestHandler.edit(req.body));
});

// handle connection delete request from frontend
app.post('/connections/remove', (req, res) => {
    res.json(connectionRequestHandler.remove(req.body));
});

// handle get user's connections request from frontend
app.post('/connections/update', (req, res) => {
    res.json(connectionRequestHandler.update(req.body));
})

// handle received webhook
app.post('/webhooks/:id', (req, res) => {
    webhookPayloadHandler(req.body, req.params.id);
});

// catch 404 and forward to error handler
app.use(function(req, res, next) {
    next(createError(404));
});

// error handler
app.use(function(err, req, res, next) {
    // set locals, only providing error in development
    res.locals.message = err.message;
    res.locals.error = req.app.get('env') === 'development' ? err : {};

    // render the error page
    res.status(err.status || 500);
    res.json({
        message: err.message,
        error: err
    });
});

module.exports = app;
