const https = require('https');
const getAction = require('./connection_actions/action-dict');
const getParser = require('./payload_parsers/parser-dict');
const validator = require('./connection-validator');
const database = require('./database-wrapper');

function handleWebhookPayload(payload, id){
    // id is from url payload was sent to

    // get webhook from database
    let webhook = database.getConnection(id);

    // get action, providerParser, and receiverParser
    let action = getAction(webhook.action);
    let provider = getParser(webhook.prov);
    let receiver = getParser(webhook.rec);
    let providerSettings = webhook.provSet;
    let receiverSettings = webhook.recSet;

    // check if connection is valid
    if (validator(action, provider, receiver, providerSettings, receiverSettings)) {
        const {data, options} = action.apply(payload, provider, receiver, providerSettings, receiverSettings);
        sendPayload(data, options);
    }
}

function sendPayload(data, options) {
    const req = https.request(options, res => {
        console.log(`statusCode: ${res.statusCode}`);

        res.on('data', d => {
            process.stdout.write(d)
        });
    })

    req.on('error', error => {
        console.error(error)
    });

    req.write(data);
}

module.exports = handleWebhookPayload;